@@ -68,9 +68,10 @@
 //! ```
 
 #![no_std]
-#![feature(llvm_asm)]
+#![feature(asm_experimental_arch)]
 #![cfg_attr(not(target_arch = "msp430"), feature(core_intrinsics))]
 
+use core::arch::asm;
 use core::cell::UnsafeCell;
 use core::fmt;
 
@@ -630,52 +631,44 @@ macro_rules! atomic_int {
         impl AtomicOperations for $int_type {
             #[inline(always)]
             unsafe fn atomic_store(dst: *mut Self, val: Self) {
-                llvm_asm!(concat!("mov", $asm_suffix, " $1, $0")
-                    :: "*m"(dst), "ir"(val) : "memory" : "volatile");
+                asm!(concat!("mov", $asm_suffix, " {1}, 0({0})"), in(reg) dst, in(reg) val);
             }
 
             #[inline(always)]
             unsafe fn atomic_load(dst: *const Self) -> Self {
                 let out;
-                llvm_asm!(concat!("mov", $asm_suffix, " $1, $0")
-                    : "=r"(out) : "*m"(dst) : "memory" : "volatile");
+                asm!(concat!("mov", $asm_suffix, " @{0}, {1}"), in(reg) dst, lateout(reg) out);
                 out
             }
 
             #[inline(always)]
             unsafe fn atomic_add(dst: *mut Self, val: Self) {
-                llvm_asm!(concat!("add", $asm_suffix, " $1, $0")
-                    :: "*m"(dst), "ir"(val) : "memory" : "volatile");
+                asm!(concat!("add", $asm_suffix, " {1}, 0({0})"), in(reg) dst, in(reg) val);
             }
 
             #[inline(always)]
             unsafe fn atomic_sub(dst: *mut Self, val: Self) {
-                llvm_asm!(concat!("sub", $asm_suffix, " $1, $0")
-                    :: "*m"(dst), "ir"(val) : "memory" : "volatile");
+                asm!(concat!("sub", $asm_suffix, " {1}, 0({0})"), in(reg) dst, in(reg) val);
             }
 
             #[inline(always)]
             unsafe fn atomic_and(dst: *mut Self, val: Self) {
-                llvm_asm!(concat!("and", $asm_suffix, " $1, $0")
-                    :: "*m"(dst), "ir"(val) : "memory" : "volatile");
+                asm!(concat!("and", $asm_suffix, " {1}, 0({0})"), in(reg) dst, in(reg) val);
             }
 
             #[inline(always)]
             unsafe fn atomic_clear(dst: *mut Self, val: Self) {
-                llvm_asm!(concat!("bic", $asm_suffix, " $1, $0")
-                    :: "*m"(dst), "ir"(val) : "memory" : "volatile");
+                asm!(concat!("bic", $asm_suffix, " {1}, 0({0})"), in(reg) dst, in(reg) val);
             }
 
             #[inline(always)]
             unsafe fn atomic_or(dst: *mut Self, val: Self) {
-                llvm_asm!(concat!("bis", $asm_suffix, " $1, $0")
-                    :: "*m"(dst), "ir"(val) : "memory" : "volatile");
+                asm!(concat!("bis", $asm_suffix, " {1}, 0({0})"), in(reg) dst, in(reg) val);
             }
 
             #[inline(always)]
             unsafe fn atomic_xor(dst: *mut Self, val: Self) {
-                llvm_asm!(concat!("xor", $asm_suffix, " $1, $0")
-                    :: "*m"(dst), "ir"(val) : "memory" : "volatile");
+                asm!(concat!("xor", $asm_suffix, " {1}, 0({0})"), in(reg) dst, in(reg) val);
             }
         }
 
